

    static bool trace_api_calls = true;
    static int trace_call_depth = NULL;
    
    /////////////////////////////////////////
    //
    //  Some simple predefined constants
    //  and macro substitutions 
    //  used to format the output strings
    //
    #define DBG_OUTPUT_PREFIX                     "   "
    #define DBG_CODE_LOCATION_STRING              __FILE__, __FUNCTION__, __LINE__
    #define DBG_OUTPUT_STRING                     "%s>%s< %s(){ @%i: %s }"
    #define DBG_OUTPUT_FORMAT(prefix, message)    DBG_OUTPUT_STRING, ((prefix == "") ? "" : prefix + " "), DBG_CODE_LOCATION_STRING, message
    
    // The actual output macro, printing to the expert-journal    
    #define DBG_MSG_MQLAPI(x)                       printf((trace_api_calls) ? "%*s%s" : "", trace_call_depth * 2, "", StringFormat(DBG_OUTPUT_PREFIX + DBG_OUTPUT_PREFIX + DBG_OUTPUT_FORMAT("", (x))))
    
    // Macro formatting the output and calling the actual MQL-API-Function
    #define DBG_MSG_MQLFUNC(x)                      DBG_MSG_MQLAPI(StringFormat((trace_api_calls) ? "MQL5-API Function => %s() [void]" : "", #x)); x

    // Same as above, but supporting a return value given by the MQL-API-Function
    #define DBG_MSG_MQLFUNC_RETURN(x, y)            dbg_mql_api_retval.msg(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat(DBG_OUTPUT_PREFIX + DBG_OUTPUT_PREFIX + DBG_OUTPUT_FORMAT("", (StringFormat("%s => %s() %35s", "MQL5-API Function", #x, "%s"))))), y) * x


    /////////////////////////////////////////
    //
    // Strip macro substitution strings
    //
    //  This function sould not be needed in 
    //  the first place, but evolved during 
    //  error searching.
    //
    const string strip_mql_api_trace(const string p_in)
    {
        if(StringFind(p_in, "dbg_mql_api_retval") > -1)
        { 
            string tmp = StringSubstr(p_in, StringFind(p_in, "*", StringFind(p_in, "MQL5-API Function")) + 1);
            StringTrimLeft(tmp);
            StringTrimRight(tmp);
            return(StringSubstr(tmp, 0, StringLen(tmp) - 1));
        }
        return(p_in);
    }


    /////////////////////////////////////////
    //
    // Enumerations and unknown types
    // 
    //  A template function to "stringify"
    //  "any"-value and return the string
    //  to print it to the yournal
    //
    //  This function is called by 
    //  macros to convert the input into 
    //  a defined string.
    //
    template <typename T>
    const string    var_out(const string name, T val, const int shift = 0, const string prefix = "", const int offset = 0)
    { return(StringFormat("%s[xxx]  %-" + IntegerToString(60 - ((offset < 60) ? offset : NULL)) + "s = '%s'", prefix, name, typename(val))); }


    /////////////////////////////////////////
    //
    //  An insertion object to enable 
    //  code execution after an MQL-API-Call
    //  was carried out.
    //
    //  Main idea is to utilize an overloaded
    //  operator from this object after the member
    //  function msg() has been called, which returns 
    //  a pointer to "this".
    //
    static class pass_through
    { public:
        string _type;
        string _msg;
        pass_through() : _msg (NULL)                            { };
        pass_through(const pass_through& s)                     { _msg = s._msg; _type = s._type; }
        pass_through* msg(const string in, const string t_in)   { 
                                                                    /////////////////////////////////////////
                                                                    //
                                                                    //  Initialize an MQL-API-Call
                                                                    //  
                                                                    //  This happens before the actual call
                                                                    //  to the MQL-API is performed.
                                                                    //
                                                                    //  Set the formatting string
                                                                    //  containing details about the call:
                                                                    //      - File
                                                                    //      - Line number in file
                                                                    //      - Function-Name to be called
                                                                    //      - expected return value
                                                                    //
                                                                    _type = t_in; 
                                                                    _msg = in; 

                                                                    // Clear current error state
                                                                    ResetLastError(); 
                                                                    
                                                                    // Return pointer to this object, so the 
                                                                    // overloaded operator can be called after the 
                                                                    // MQL-API-Call has happend.
                                                                    return(GetPointer(this)); 
                                                                }
        template <typename T>
        T operator*(T in)                                       {
                                                                    if(trace_api_calls)
                                                                    {
                                                                        const int err_no = GetLastError();
                                                                        const string _typename = typename(T);

                                                                        /////////////////////////////////////////
                                                                        //
                                                                        //  After the MQL-API-Call, print 
                                                                        //  details about this call to the 
                                                                        //  expert journal.
                                                                        //
                                                                        //  _msg member variable contains 
                                                                        //  a "format-string" which was set my the 
                                                                        //  call to the member function .msg();
                                                                        printf("%s", 
                                                                            StringFormat(_msg, 
                                                                                (err_no != ERR_SUCCESS) ? 
                                                                                    StringFormat("Error: %i", err_no) : 
                                                                                    (   ((StringFind(_type, "void") == -1) && (StringFind(_type, _typename) == -1)) ? 
                                                                                            StringFormat("Type mismatch. Expected: %s, Got: %s", _type, _typename) : 
                                                                                            var_out("returns", in, NULL, "", 45)
                                                                                    )));
                                                                        _msg = NULL;
                                                                    }
                                                                    
                                                                    // Return the resulting value from the MQL-API function call
                                                                    return(in);
                                                                }
    } dbg_mql_api_retval;




    /////////////////////////////////////////
    //
    //  These macros have been defined
    //  to help in finding the root cause 
    //  of the issue at hand.
    //
    //  These are copies of the original, so 
    //  tinkering with them still leaves the 
    //  references in tact. - 
    //
    #define _DBG_OUTPUT_PREFIX                          "  "
    #define _DBG_CODE_LOCATION_STRING                   __FILE__, __FUNCTION__, __LINE__
    #define _DBG_CODE_LOCATION(x)                       StringFormat("%s %s %s %i", (x), _DBG_CODE_LOCATION_STRING)
    #define _DBG_OUTPUT_STRING                          "%s >%s< %s(){ @%i: %s }"
    #define _DBG_OUTPUT_FORMAT(prefix, message)         _DBG_OUTPUT_STRING, (((prefix) == "") ? "" : (prefix)), _DBG_CODE_LOCATION_STRING, (message)


    /////////////////////////////////////////
    //
    //  A helper macro to 
    //  support the error finding process
    //
    //  A so called stringify-macro, which will 
    //  put '"' around the given input.
    //
    #define _DBG_HLP_TO_STR(x)                          (#x)


    /////////////////////////////////////////
    //
    // This macro helps by calculating the 
    //  indentation for the output required.
    //    
    #define _DBG_MSG_SHIFT                              (StringLen(                                                                                         \   // Calculate the length of the output
                                                            StringFormat("%*s%s",                                                                           \   // Formatting string
                                                                trace_call_depth * 2, "",                                                                   \   // Indentation based on calldepth from function tracing
                                                                StringFormat(_DBG_OUTPUT_PREFIX + _DBG_OUTPUT_PREFIX + _DBG_OUTPUT_FORMAT("", ("")))        \   // Output string template
                                                            )) - 2)


    /////////////////////////////////////////
    //
    // This is the actual print-output macro 
    //  used to print to the expert journal.
    //
    #define _DBG_MSG_MQLAPI(x)                          printf((trace_api_calls) ? "%*s%s" : "", trace_call_depth * 2, "", StringFormat(DBG_OUTPUT_PREFIX + DBG_OUTPUT_PREFIX + DBG_OUTPUT_FORMAT("", (x))))


    /////////////////////////////////////////
    //
    //  These macros perform the actual 
    //  MQL-API-Function call
    //
    //  First they set up the output
    //  and then they perform the function call
    //  with given parameters by the user itself.
    //    
    #define _DBG_MSG_MQLFUNC(x)                         _DBG_MSG_MQLAPI(StringFormat((trace_api_calls) ? "MQL5-API Function => %s() [void]" : "", #x)); x
    #define _DBG_MSG_MQLFUNC_RETURN(x, y)               dbg_mql_api_retval.msg(                                                                                         \       // This is the initial setup-call to the "pass-through" object.
                                                            StringFormat("%*s%s", trace_call_depth * 2, "",                                                             \       // Here the output format string gets set
                                                                StringFormat(_DBG_OUTPUT_PREFIX + _DBG_OUTPUT_PREFIX +                                                  \       // This is the prefix, which gets added to the format string.
                                                                                _DBG_OUTPUT_FORMAT("", (StringFormat("%s => %s() %35s", "MQL5-API Function", #x, "%s")))\       // This macro will expand and add specific call details. Expanded result:
                                                                                )                                                                                       \       //      "%s >%s< %s(){ @%i: %s }", "", __FILE__, __FUNCTION__, __LINE__, "MQL5-API Function => >>here the macro parameter x stringified<<() %s"
                                                                ), y) * x                                                                                                       // At the end of this macro, the MQL-API-Function name, represented by x.
    
    // This is a testing variant of the above macro
    // I have tried to remove the actual expansion,
    //  hoping, it would solve the primary issue.
    //  It does not solve the issue.
    #define TEST_DBG_MSG_MQLFUNC_RETURN(x, y)           dbg_mql_api_retval.msg(                                                                                         \       // This is the initial setup-call to the "pass-through" object.
                                                            StringFormat("%*s%s", trace_call_depth * 2, "",                                                             \       // Here the output format string gets set
                                                                StringFormat(_DBG_OUTPUT_PREFIX + _DBG_OUTPUT_PREFIX +                                                  \       // This is the prefix, which gets added to the format string.
                                                                                _DBG_OUTPUT_FORMAT("", (StringFormat("%s => %s() %35s", "MQL5-API Function", x, "%s"))) \       // This macro will expand and add specific call details. Expanded result:
                                                                                )                                                                                       \       //      "%s >%s< %s(){ @%i: %s }", "", __FILE__, __FUNCTION__, __LINE__, "MQL5-API Function => >>here the macro parameter x<<() %s"
                                                                ), y) * 


    
    /////////////////////////////////////////
    //
    //  Output format helper macros
    //
    
    // Convert a macro parameter into a string, 
    // shift/indent the string according to the current trace depth
    #define _DBG_STR_VAR(x)                             (var_out(strip_mql_api_trace(#x), (x), _DBG_MSG_SHIFT))

    // Actually print a sting to the expert journal
    // Apply relevant formatting operations
    #define _DBG_MSG(x)                                 printf("%*s%s", trace_call_depth * 2, "", StringFormat(_DBG_OUTPUT_PREFIX + _DBG_OUTPUT_PREFIX + _DBG_OUTPUT_FORMAT("", (x))))

    // Convert and print a value of (almost) any type to the expert journal
    #define _DBG_MSG_VAR(x)                             _DBG_MSG(_DBG_STR_VAR((x)))

    // This is the same macro as above, but pre-expanded to find the error in the expansion process.
    #define __DBG_MSG_VAR(x)                            _DBG_MSG((var_out(strip_mql_api_trace(#x), (x), _DBG_MSG_SHIFT)))

    // This is the fully expanded macro as far it is relevant 
    // for diagnosing the input being a macro itself.
    // The issue here is that x is a macro and should get expanded only once, but it is expanded multiple times
    // The result is shown below.
    #define ___DBG_MSG_VAR(x)                           printf("%*s%s", trace_call_depth * 2, "", StringFormat(_DBG_OUTPUT_PREFIX + _DBG_OUTPUT_PREFIX + _DBG_OUTPUT_STRING, "", _DBG_CODE_LOCATION_STRING, var_out(strip_mql_api_trace(#x), (x), _DBG_MSG_SHIFT)))


    /////////////////////////////////////////
    //
    //  These macros replace the original 
    //  MQL function call, substituting 
    //  the "pass_through" object to 
    //  enable insertion of debugging details before
    //  the actual function call, and evaluating and
    //  printing these details after the function call.
    //  
    //  For this to be achieved, the object needs to be 
    //  placed before the function is called. 
    //  Examples can be seen below, at the end of this listing.
    //
    #define TimeCurrent                                 TEST_DBG_MSG_MQLFUNC_RETURN("TimeCurrent", "datetime") TimeCurrent
    #define FileClose                                   _DBG_MSG_MQLFUNC(FileClose)
    #define FileOpen                                    _DBG_MSG_MQLFUNC_RETURN(FileOpen, "int")
    #define FileSize                                    _DBG_MSG_MQLFUNC_RETURN(FileSize, "ulong")

    // Macro debugging helper
    #define TO_STRING(x)        #x





/////////////////////////////////////////
//
//  A helper object for debugging 
//  purposes. 
//
//  In fact, it is not required anymore,
//  as this has already served its purpose
//  and is left here for completness.
//
//  Original issue was the order of execution.
//
//  The solution to this was to use an 
//  operator thats ranked high enough to be 
//  called early enough to give proper execution
//  order in (hopefully) all cases.
//
class CTestObj
{   
    public:
    CTestObj()
    { };
    CTestObj(const CTestObj& p_in)
    { };
    
    const bool operator==(int p_in) const { printf("%s", "Call operator==()"); return(true); }
} test_obj;


CTestObj* Test_Func(const int f_h)
{ printf("%s", "Call Test_Func()"); return(GetPointer(test_obj)); };



void OnInit()
{

            /////////////////////////////////////////
            //
            //  Outer most printf() takes 7 parameters.
            //
            //      1. Format-string
            //      2. Indentation integer and 3. an empty placeholder
            //      4. (string)__FILE__
            //      5. (string)__FUNCTION__
            //      6. (int)__LINE__
            //      7. the actual output string produced by the function var_out()
            //
            //  printf("%*s%s", 
            //      trace_call_depth * 2, "", 
            //      StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", "", __FILE__, __FUNCTION__, __LINE__, 
                                
                                /////////////////////////////////////////
                                //
                                // The function var_out takes 
                                //  - 1. Name of the "value" to be stringified
                                //  - 2. the actual value to stringify
                                //  - 3. an integer for right shifting the output
                                //  - 4. a prefix to add to the output string
                                //  - 5. an offset for left-justification
                                //
                                //  The function "strip_mql_api_trace" should not be required.
                                //
            //                  var_out(
                                    //  1. parameter
                                    //
                                    // The input parameter to "strip_mql_api_trace"
                                    // should for this case be "TimeCurrent", but for some reason it gets expanded, which should not be...
                                    //  But to initially deal with this, I have created the function "strip_mql_api_trace", which stripps the 
                                    //  unneccessary stuff from the string and returns only the last part, here "TimeCurrent()".
                                    //
                                    // NOTE: 
                                    //      As can be seen by the input string, the preprocessor stringification is faulty, it should not
                                    //      strip out white-spaces nor remove relevant '"' from the original input, but excape these charactersproperly.
            //                      strip_mql_api_trace("dbg_mql_api_retval.msg(StringFormat(%*s%s,trace_call_depth*2,,StringFormat(  +  +%s >%s< %s(){ @%i: %s },((()==)?:()),__FILE__,__FUNCTION__,__LINE__,((StringFormat(%s => %s() %35s,MQL5-API Function,TimeCurrent,%s))))),datetime)*TimeCurrent()"), 
    
                                    //  2. parameter
                                    //
                                    // This input parameter is the actual value, where the function call to MQL-API is performed 
                                    //  and results in the return value being passed to the function "var_out" as parameter.
                                    //
                                    //  This process is split in multiple steps.
                                    //      1. call "dbg_mql_api_retval.msg" with the relevant details to save them 
                                    //          for later printing. (MQL-API-Function name and "return value types supported by this function)
                                    //          These details are provided by the replacement-macros, in this case "TimeCurrent"
                                    //      2. return a pointer to itself, so the overloaded operatro will be called.
            //                      (dbg_mql_api_retval.msg(
                                        // This stringFormat is actually a macro, descried above.
                                        // For reference please see: _DBG_MSG_MQLFUNC_RETURN
                                        // Beginning by the first StringFormat from that macro.
                                        // I will spare deeper explanations on this, as it is described above already.
            //                          StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), 
            //                          "datetime")
                                    // Here is the first repetition, that is wrong.
                                    // The multiplication-opertator "*" is overloaded by the object "dbg_mql_api_retval"
                                    // And therefore should now call
            //                      * dbg_mql_api_retval.msg(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), "datetime") 
                                    // And annother repition follows
            //                      * dbg_mql_api_retval.msg(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), "datetime") 
                                    // This is the actual function call which should have been the only mentioning, and the only "multiplication"
            //                      * TimeCurrent()), 
                                    
                                    // 3. parameter
                                    //
                                    // Calculate the relevant right shifting for the output string and pass it as value to the function "var_out"
            //                      (StringLen(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((""))))) - 2)
                                    
                                    // 4. and 5. parameter are ommited
                                    //
                                    //  The defaults apply, as given by the functions signature.
                                    //  they are: "" and 0
            //                      ) // Closingvar_out
    
            //          ) // Closing StringFormat
    
            //      ); // Closing printf



    ////////////////////////////////////////////////////////////////////////////////
    //
    //  Possible error in the preprocessor stage of the compiler.
    //
    //  For some reason, the preprocessor keeps substituting the same macro
    //  over and over again, though, it should stop after first substitution.
    //
    //  To narrow down the issue, all relevant code is provided here.
    //
    //      Concerning the macro ___DBG_MSG_VAR()
    //
    //      Expected behaviour would be, and this was verified with g++ compiler
    //      The parameter passed to the macro ___DBG_MSG_VAR() should only expand once.
    //      Even if it is another macro, which it is in this case.
    //
    //      But the result shows multitude of substitutions of the same macro, creating a
    //      substitution loop, which is in contrast to g++ compiler for sure not the expected
    //      or wanted behaviour.
    //
    //  Here is what actually is produced by the preprosessor stage:
    //
    //
    //  Outer most printf() takes 7 parameters.
    //
    //      1. Format-string
    //      2. Indentation integer and 3. an empty placeholder
    //      4. (string)__FILE__
    //      5. (string)__FUNCTION__
    //      6. (int)__LINE__
    //      7. the actual output string produced by the function var_out()
    //
    //  printf("%*s%s", 
    //      trace_call_depth * 2, "", 
    //      StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", "", __FILE__, __FUNCTION__, __LINE__, 
    //
                        /////////////////////////////////////////
                        //
                        // The function var_out takes 
                        //  - 1. Name of the "value" to be stringified
                        //  - 2. the actual value to stringify
                        //  - 3. an integer for right shifting the output
                        //  - 4. a prefix to add to the output string
                        //  - 5. an offset for left-justification
                        //
                        //  The function "strip_mql_api_trace" should not be required.
                        //
    //                  var_out(
                            //  1. parameter
                            //
                            // The input parameter to "strip_mql_api_trace"
                            // should for this case be "TimeCurrent", but for some reason it gets expanded, which should not be...
                            //  But to initially deal with this, I have created the function "strip_mql_api_trace", which stripps the 
                            //  unneccessary stuff from the string and returns only the last part, here "TimeCurrent()".
                            //
                            // NOTE: 
                            //      As can be seen by the input string, the preprocessor stringification is faulty, it should not
                            //      strip out white-spaces nor remove relevant '"' from the original input, but excape these charactersproperly.
    //                      strip_mql_api_trace("dbg_mql_api_retval.msg(StringFormat(%*s%s,trace_call_depth*2,,StringFormat(  +  +%s >%s< %s(){ @%i: %s },((()==)?:()),__FILE__,__FUNCTION__,__LINE__,((StringFormat(%s => %s() %35s,MQL5-API Function,TimeCurrent,%s))))),datetime)*TimeCurrent()"), 
    //
                            //  2. parameter
                            //
                            // This input parameter is the actual value, where the function call to MQL-API is performed 
                            //  and results in the return value being passed to the function "var_out" as parameter.
                            //
                            //  This process is split in multiple steps.
                            //      1. call "dbg_mql_api_retval.msg" with the relevant details to save them 
                            //          for later printing. (MQL-API-Function name and "return value types supported by this function)
                            //          These details are provided by the replacement-macros, in this case "TimeCurrent"
                            //      2. return a pointer to itself, so the overloaded operatro will be called.
    //                      (dbg_mql_api_retval.msg(
                                // This stringFormat is actually a macro, descried above.
                                // For reference please see: _DBG_MSG_MQLFUNC_RETURN
                                // Beginning by the first StringFormat from that macro.
                                // I will spare deeper explanations on this, as it is described above already.
    //                          StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), 
    //                          "datetime")
                            // Here is the first repetition, that is wrong.
                            // The multiplication-opertator "*" is overloaded by the object "dbg_mql_api_retval"
                            // And therefore should now call
    //                      * dbg_mql_api_retval.msg(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), "datetime") 
                            // And annother repition follows
    //                      * dbg_mql_api_retval.msg(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), "datetime") 
                            // This is the actual function call which should have been the only mentioning, and the only "multiplication"
    //                      * TimeCurrent()), 
    //
                            // 3. parameter
                            //
                            // Calculate the relevant right shifting for the output string and pass it as value to the function "var_out"
    //                      (StringLen(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((""))))) - 2)
    //
                            // 4. and 5. parameter are ommited
                            //
                            //  The defaults apply, as given by the functions signature.
                            //  they are: "" and 0
    //                      ) // Closingvar_out
    //
    //          ) // Closing StringFormat
    //
    //      ); // Closing printf
    //
    //
    //
    //
    //  And this is the result from the g++ compiler, using the -E option.
    //
    //  Outer most printf() takes 7 parameters.
    //
    //      1. Format-string
    //      2. Indentation integer and 3. an empty placeholder
    //      4. (string)__FILE__
    //      5. (string)__FUNCTION__
    //      6. (int)__LINE__
    //      7. the actual output string produced by the function var_out()
    //
    //  printf("%*s%s", 
    //      trace_call_depth * 2, "", 
    //      StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", "", __FILE__, __FUNCTION__, __LINE__, 
    //
                        /////////////////////////////////////////
                        //
                        // The function var_out takes 
                        //  - 1. Name of the "value" to be stringified
                        //  - 2. the actual value to stringify
                        //  - 3. an integer for right shifting the output
                        //  - 4. a prefix to add to the output string
                        //  - 5. an offset for left-justification
                        //
                        //  The function "strip_mql_api_trace" should not be required.
                        //
    //                  var_out(
                            //  1. parameter
                            //
                            // The input parameter to "strip_mql_api_trace"
                            // should for this case be "TimeCurrent", but for some reason it gets expanded, which should not be...
                            //  But to initially deal with this, I have created the function "strip_mql_api_trace", which stripps the 
                            //  unneccessary stuff from the string and returns only the last part, here "TimeCurrent()".
                            //
                            // NOTE: 
                            //      Here the input parameter is actually the correct result as I would have expected it.
    //                      strip_mql_api_trace("TimeCurrent()"), 
    //
                            //  2. parameter
                            //
                            // This input parameter is the actual value, where the function call to MQL-API is performed 
                            //  and results in the return value being passed to the function "var_out" as parameter.
                            //
                            //  This process is split in multiple steps.
                            //      1. call "dbg_mql_api_retval.msg" with the relevant details to save them 
                            //          for later printing. (MQL-API-Function name and "return value types supported by this function)
                            //          These details are provided by the replacement-macros, in this case "TimeCurrent"
                            //      2. return a pointer to itself, so the overloaded operatro will be called.
    //                      (dbg_mql_api_retval.msg(
                                // This stringFormat is actually a macro, descried above.
                                // For reference please see: _DBG_MSG_MQLFUNC_RETURN
                                // Beginning by the first StringFormat from that macro.
                                // I will spare deeper explanations on this, as it is described above already.
    //                          StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((StringFormat("%s => %s() %35s", "MQL5-API Function", "TimeCurrent", "%s"))))), 
    //                          "datetime")
                            // No further repetitions happen, as I would have expected it to be.
    //                      * TimeCurrent()), 
    //
                            // 3. parameter
                            //
                            // Calculate the relevant right shifting for the output string and pass it as value to the function "var_out"
    //                      (StringLen(StringFormat("%*s%s", trace_call_depth * 2, "", StringFormat("  " + "  " + "%s >%s< %s(){ @%i: %s }", ((("") == "") ? "" : ("")), __FILE__, __FUNCTION__, __LINE__, ((""))))) - 2)
    //
                            // 4. and 5. parameter are ommited
                            //
                            //  The defaults apply, as given by the functions signature.
                            //  they are: "" and 0
    //                      ) // Closingvar_out
    //
    //          ) // Closing StringFormat
    //
    //      ); // Closing printf
    //
    //
    //  I have aligned the outputs, it is relevant to scroll all the way to the 
    //  right to see the difference in results.
    //
    //  Anyways, the code produced by the MQL-Preprocessor lets the program crash harshly,
    //  resulting in:
    //
    //  Access violation at 0x00000273C7E21008 read to 0xFFFFFFFFFFFFFFFF in 'LibDebug.ex5'
    //
    //
    //  Additionally, I noticed the macro-operator # for stringifying, does not work properly, 
    //  as it removes all '"' instead of escaping them properly, also whitespaces are being removed, 
    //  giving false results in the outcome.
    //
    //
    //  As a note:
    //      It was quite difficult to track down this issue, as there is very limited way of 
    //      debugging macros in the first place, and MQL-Compiler not providing output of the
    //      produced results of the preprosessor-stage makes things even harder.
    //
    //      I tried best by applying the stringify-macro to the end-results, but this still could 
    //      influence the interpretation of the results, as to above mentioned issue with the 
    //      stringify-operator.
    //
    //  This (assumed) bug in the prerocessor stage of the compiler hinders me on finishing 
    //  the project, which can be found here on code-base:
    //
    //      https://www.mql5.com/en/code/32071
    //


    /////////////////////////////////////////
    //
    // For debugging purposes
    // 
    // This will (or should) show the resulting, 
    //  expanded code produced by the preprocessor.
    //
    string test1 = TO_STRING(___DBG_MSG_VAR(TimeCurrent()));


    /////////////////////////////////////////
    //
    // This is the usage of the manually 
    //  expanded macro, that should not throw an
    //  exception.
    //
    //  The less manual expansion is done, the 
    //  more often the repetition happens,
    //
    ___DBG_MSG_VAR(TimeCurrent());


    /////////////////////////////////////////
    // 
    // More repetitions....
    //
    string test2 = TO_STRING(__DBG_MSG_VAR(TimeCurrent()));
    string test3 = TO_STRING(_DBG_MSG_VAR(TimeCurrent()));





    /////////////////////////////////////////
    // 
    // If you disable the above four 
    //  lines of code, you can see how the 
    //  call tracing actually works, by
    //  letting below code run.
    //

    int f_h = FileOpen("some_file.txt", FILE_READ | FILE_COMMON);
    ulong size_of_file = FileSize(f_h);

    if(FileSize(f_h) == 5)
    {
        printf("Filesize != 0");
    }

    if(FileSize(f_h) != 5)
    {
        printf("Filesize != 5");
    }
    
    if(5 == FileSize(f_h))
    {
        printf("5 == FileSize");
    }
    
    if(5 != FileSize(f_h))
    {
        printf("5 != FileSize");
    }
    
    FileClose(f_h);
};



/*

//+------------------------------------------------------------------+
//|                                                     LibDebug.mq5 |
//|                                             Licensed under GPLv2 |
//|                                       https://www.freie-netze.de |
//+------------------------------------------------------------------+
#property copyright "GPLv2"
#property link      "https://www.gnu.org/licenses/old-licenses/gpl-2.0.html"
#property version   "1.00"








///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Include debugging library
//
//  Select operational mode:
//
//  - #define LIB_DEBUG
//      -> This will disable LIB_PERF_PROFILING and enable DEBUGGING extensions
//
//  - #define LIB_PERF_PROFILING
//      -> This will enable PERF_PROFILING macros. Only available in "runtime"-mode compilation.
//          If trying to measure these metrics while "_DEBUG" or "LIB_DEBUG" is defined will not work.
//
//  Define these before you include the library, as they define the included parts of the library.
//
//      LIB_DEBUG is superior to LIB_PERF_PROFILING and will disable LIB_PERF_PROFILING.
//
//  Debugging library supports output logging to file. This feature can be enabled by defining
//  "LIB_DEBUG_LOGFILE" as shown below. (Automatically also defines LIB_DEBUG)
//      Logs will be written to Common/* directory.
//
//      This feature supports debug logging within optimizer runs of the program.
//
//  Additionally the expert-journal output can be surpressed, separating the logs into two
//  targets. - This way logging debug output and logging terminal output is separated.
//
//  Define "LIB_DEBUG_NO_JOURNAL_OUTPUT" to disable expert-journal output from this library.
//
#define LIB_DEBUG
//#define LIB_DEBUG_LOGFILE
//#define LIB_DEBUG_NO_JOURNAL_OUTPUT
#define LIB_DEBUG_MQLAPI
#define LIB_DEBUG_MQLAPI_ALL_CALLS
#define LIB_PERF_PROFILING
#include <MQLplus/lib_debug.mqh>



///////////////////////////////////////////////////////////////////////
//
// In OnInit you can see a simple usage of the debugging macros.
//
//  We will only use some stand-alone macros for quick debugging.
//
//  DBG_MSG()
//  DBG_MSG_VAR()
//  DBG_MSG_VAR_IF()
//
//  These macros support all built-in datatypes (also in form of arrays).
//      (unsigned) char, short, int, long
//      bool, datetime, color, complex, matrix, vector, 
//      float, double,
//      string, enum, struct, class and interface as well as pointers.
//
//
//  And MQL structures:
//      MqlDateTime, MqlRates, MqlTick, MqlParam, MqlBookInfo,
//      MqlTradeRequest, MqlTradeCheckResult, MqlTradeResult,
//      MqlTradeTransaction, 
//      MqlCalendarCountry, MqlCalendarEvent, MqlCalendarValue
//

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
    DBG_MSG("Example how to use lib_debug.mqh");

    // Output details about current program
    printf("%s", DBG_STR_EX45_FILEINFO);


    EventSetTimer(10);


    // Output primitive variables
    ENUM_APPLIED_PRICE  e_val    = PRICE_CLOSE;
    char        c_val       = 1;
    uchar       uc_val      = 2;
    short       s_val       = 3;
    int         i_val       = 4;
    long        l_val       = 5;
    bool        b_val       = true;
    datetime    d_val       = TimeCurrent();
    float       flt_val     = (float)1.61;
    double      dbl_val     = 3.14;
    color       clr_val     = clrAqua;
    string      str_val     = "Some string";
    complex     cmplx_val   = { 0.1, 0.2 };
    matrix      m_val(1, 2);
    matrixf     mf_val(1.3, 2.3);
    vector      v_val       = { 0, 1, 2, 3 };
    vectorf     vf_val      = { (float)0.1, (float)1.1, (float)2.1, (float)3.1 };

    complex  c1 = { 0.1, 0.2 };
    complex  c2 = { 0.3, 0.4 };
    complex  c3 = { 0.5, 0.6 };
    complex  c4 = { 0.6, 0.7 };
    matrixc     mc_val      {{c1, c2}, {c3, c4}};
    vectorc     vc_val      = { c1, c2, c3, c4 };

    enum custom_enum
    {
        SOME_ENUM_VAL1,
        SOME_ENUM_VAL2
    };
    custom_enum my_enum = SOME_ENUM_VAL2;

    //////////////////////////////////////////////////////////////
    //
    // Limitations of DBG_MSG_VAR() macro
    //
    //  Direct resolving of enumeration types does not work.
    //
    //DBG_MSG_VAR(PRICE_CLOSE);
    //DBG_MSG_VAR((ENUM_APPLIED_PRICE)PRICE_CLOSE);
    //
    //  Instead a variable must be passed.
    //
    //ENUM_APPLIED_PRICE  e_val    = PRICE_CLOSE;
    //DBG_MSG_VAR(e_val);

    // Direct value printing
    DBG_MSG_VAR(1);
    DBG_MSG_VAR(true);
    DBG_MSG_VAR(TimeCurrent());
    DBG_MSG_VAR(clrBlue);
    DBG_MSG_VAR("Print Me");
    DBG_MSG_VAR(0.25);

    DBG_MSG_VAR(my_enum);
    DBG_MSG_VAR(e_val);
    DBG_MSG_VAR(c_val);
    DBG_MSG_VAR(uc_val);
    DBG_MSG_VAR(s_val);
    DBG_MSG_VAR(i_val);
    DBG_MSG_VAR(l_val);
    DBG_MSG_VAR(b_val);
    DBG_MSG_VAR(d_val);
    DBG_MSG_VAR(flt_val);
    DBG_MSG_VAR(dbl_val);
    DBG_MSG_VAR(clr_val);
    DBG_MSG_VAR(str_val);
    DBG_MSG_VAR(cmplx_val);
    DBG_MSG_VAR(m_val);
    DBG_MSG_VAR(mf_val);
    DBG_MSG_VAR(mc_val);
    DBG_MSG_VAR(v_val);
    DBG_MSG_VAR(vf_val);
    DBG_MSG_VAR(vc_val);

    // Example of printing objects
    call_obj_function();



    //////////////////////////////////////////////////////////////
    //
    // Conditional execution of DBG_MSG_VAR() macro
    //
    //  Use DBG_MSG_VAR_IF() as shown below.
    //
    // NOTE: The fist statement is a boolean evaluation.
    //       It is encapuslated properly and therefore can 
    //       be anything that evaluates to "true" or "false".
    //       Same rules apply as for an "if()" statement.
    
    bool printme = true;
    //DBG_MSG_VAR_IF(printme, TimeLocal());

    return(INIT_FAILED);




    // Output arrays
    int i_arr[50];
    for(int cnt = NULL; (cnt < 50) && !_StopFlag; cnt++)
    { i_arr[cnt] = MathRand(); }
    DBG_MSG_VAR(i_arr);

    matrixc mc_val1 {{c1, c2}, {c3, c4}};
    matrixc mc_val2 {{c2, c1}, {c3, c4}};
    matrixc mc_val3 {{c1, c2}, {c4, c3}};
    matrixc     mc_val_arr[3];
    mc_val_arr[0]       = mc_val1;
    mc_val_arr[1]       = mc_val2;
    mc_val_arr[2]       = mc_val3;
    DBG_MSG_VAR(mc_val_arr);

    string str_val_arr[3];
    DBG_MSG_VAR(str_val_arr);


    
    // Output MQL structures
    MqlDateTime             mql_dtm;
    MqlRates                mql_rates; 
    MqlTick                 mql_tick;
    MqlParam                mql_param;
    MqlBookInfo             mql_book;
    MqlTradeRequest         mql_trade_request;
    MqlTradeCheckResult     mql_tradecheckresult;
    MqlTradeResult          mql_trade_result;
    MqlTradeTransaction     mql_transaction;
    MqlCalendarCountry      mql_cal_cntry;
    MqlCalendarEvent        mql_cal_event;
    MqlCalendarValue        mql_cal_value;
    
    DBG_MSG_VAR(mql_dtm);
    DBG_MSG_VAR(mql_rates);
    DBG_MSG_VAR(mql_tick);
    DBG_MSG_VAR(mql_param);
    DBG_MSG_VAR(mql_book);
    DBG_MSG_VAR(mql_trade_request);
    DBG_MSG_VAR(mql_tradecheckresult);
    DBG_MSG_VAR(mql_trade_result);
    DBG_MSG_VAR(mql_transaction);
    DBG_MSG_VAR(mql_cal_cntry);
    DBG_MSG_VAR(mql_cal_event);
    DBG_MSG_VAR(mql_cal_value);



    // Output arrays of structures
    MqlDateTime             arr_mql_dtm[3];
    MqlRates                arr_mql_rates[3]; 
    MqlTick                 arr_mql_tick[3];
    MqlParam                arr_mql_param[3];
    MqlBookInfo             arr_mql_book[3];
    MqlTradeRequest         arr_mql_trade_request[3];
    MqlTradeCheckResult     arr_mql_tradecheckresult[3];
    MqlTradeResult          arr_mql_trade_result[3];
    MqlTradeTransaction     arr_mql_transaction[3];
    MqlCalendarCountry      arr_mql_cal_cntry[3];
    MqlCalendarEvent        arr_mql_cal_event[3];
    MqlCalendarValue        arr_mql_cal_value[3];

    DBG_MSG_VAR(arr_mql_dtm);
    DBG_MSG_VAR(arr_mql_rates);
    DBG_MSG_VAR(arr_mql_tick);
    DBG_MSG_VAR(arr_mql_param);
    DBG_MSG_VAR(arr_mql_book);
    DBG_MSG_VAR(arr_mql_trade_request);
    DBG_MSG_VAR(arr_mql_tradecheckresult);
    DBG_MSG_VAR(arr_mql_trade_result);
    DBG_MSG_VAR(arr_mql_transaction);
    DBG_MSG_VAR(arr_mql_cal_cntry);
    DBG_MSG_VAR(arr_mql_cal_event);
    DBG_MSG_VAR(arr_mql_cal_value);



    return(INIT_SUCCEEDED);
}



///////////////////////////////////////////////////////////////////////
//
// In OnDeinit we will use full tracing of the function call.
//
//  First we define some helpers for compile time code
//  inclusion selection. This will make it easy to enable/disable 
//  function tracing as needed.
//
//  To do this, first a macro is defined which will be used as a 
//  switch (defined = trace function) (not defined = disable tracing)
//
//  We can collect these "switches" at top of file for a better overview
//  Also we could wrap them into a condition, so that they will only be 
//  defined when debugging is enabled.
//
//  Example:
//  #ifdef LIB_DEBUG
//      #define DBG_TRACE_EXPERT_MAIN_ONDEINIT
//
//  #endif

// En/Disable function tracing by commenting following macro:
#define DBG_TRACE_EXPERT_MAIN_ONDEINIT


// Now we define the helper macros which will be used inside 
//  the functions body.

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
/////////////////////////////////////
// Function debug trace code
#ifdef DBG_TRACE_EXPERT_MAIN_ONDEINIT
    #undef DBG_TRACE_EXPERT_MAIN_ONDEINIT
    #define DBG_TRACE_EXPERT_MAIN_ONDEINIT(x) x
    #define DBG_TRACE_EXPERT_MAIN_ONDEINIT_RETURN       DBG_MSG_TRACE_RETURN

#else
    #define DBG_TRACE_EXPERT_MAIN_ONDEINIT(x)
    #define DBG_TRACE_EXPERT_MAIN_ONDEINIT_RETURN       DBG_MSG_NOTRACE_RETURN

#endif
/////////////////////////////////////
void OnDeinit(const int reason)
{
    /////////////////////////////////////
    //
    // TOP of function body
    //
    // Here the tracing code gets included, this is mandatory for tracing to work.
    // The switching macro can also be used to include additional 
    // debugging code, if any is required, see example below.
    DBG_TRACE_EXPERT_MAIN_ONDEINIT(
        DBG_MSG_TRACE_BEGIN;
        
        // Additional details can be added here, as required.
        // For example to trace the inputs to this function, we can add
        DBG_MSG_VAR(reason);
    
        // Or whatever we would like to add.
        );
        
        // Additionally the macro "PERF_COUNTER_BEGIN" must be added.
        // It is mandatory to complete the "return" macro, irrespective of it
        // being used or not. (Any insertions will be removed for runtime builds,
        // if "LIB_PERF_PROFILING" is not defined)
        PERF_COUNTER_BEGIN;
    
    //
    // This concludes the functions head
    //  control code required for tracing.
    //  If tracing is disabled for this function,
    //  all additional code will be stripped at 
    //  compile time.
    //
    //  Now follows the actual functions body:
    /////////////////////////////////////


    // Local init
    
        int     some_int_val    = 3;
        string  some_string     = "I dont know";

        /////////////////////////////////////
        //
        // As an example, here the local variables 
        //  are printed, but only if tracing is enbaled.
        //  In contrast to the usage of macros in OnInit, where
        //  the macros were not wrapped in the 
        //  Trace-Enable-Disable macro.
        //
        DBG_TRACE_EXPERT_MAIN_ONDEINIT(
            DBG_MSG_VAR(some_int_val);
            DBG_MSG_VAR(some_string);
            );


    // Destroy timer

        EventKillTimer();




    /////////////////////////////////////
    //
    // EXITING/RETURNING from the fucntions body.
    //
    // A return statement is mandatory, no matter the functions signature.
    // This will inform the debug library about the functions ending/exiting. 
    //
    //  The special function "OnDeinit" will auto-resolve the deinit reason
    //  and show the state of the _StopFlag/IsStopped() variable.
    //  This is automatically inserted and needs no extra care taking.
    //
    DBG_TRACE_EXPERT_MAIN_ONDEINIT_RETURN;
}




///////////////////////////////////////////////////////////////////////
//
// In OnTick we will use full tracing of the function call as 
//  shown with OnDeinit, and we will integrate global performance 
//  counters as well. This will give output of the runtime used by 
//  this function.
//
//  Since the tracing has already been exlained, here is a 
//  stripped version, showing the raw requirements for a full trace.
//  
//  Short outline. 
//  - First define the enable/disable macros.
//  - Then include the head required inside the body.
//  - Use the return macro at all exit points of the function.
//
//  To enable tracing for this function, same procedure as 
//  with OnDeinit applies.
//  Again, this could be collected at top of file for better oversight.

// Turn on tracing
#define DBG_TRACE_EXPERT_MAIN_ONTICK


//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
/////////////////////////////////////
// Function debug trace code
#ifdef DBG_TRACE_EXPERT_MAIN_ONTICK
    #undef DBG_TRACE_EXPERT_MAIN_ONTICK
    #define DBG_TRACE_EXPERT_MAIN_ONTICK(x) x
    #define DBG_TRACE_EXPERT_MAIN_ONTICK_RETURN         DBG_MSG_TRACE_RETURN

#else
    #define DBG_TRACE_EXPERT_MAIN_ONTICK(x)
    #define DBG_TRACE_EXPERT_MAIN_ONTICK_RETURN         DBG_MSG_NOTRACE_RETURN

#endif
/////////////////////////////////////
void OnTick()
{
DBG_TRACE_EXPERT_MAIN_ONTICK(
    DBG_MSG_TRACE_BEGIN;    
    );
PERF_COUNTER_BEGIN;

    // Local init
    int some_var = MathRand();


    // Some random check operation
    if((some_var % 2) == 0)
    {
        call_sub_function();
        call_sub2_function();
        call_obj_function();
    }


    // Some eventual exit at annother point inside the functions body
    if((some_var % 2048) == 0)
    { DBG_TRACE_EXPERT_MAIN_ONTICK_RETURN; }


    // Return
    DBG_TRACE_EXPERT_MAIN_ONTICK_RETURN;
}






///////////////////////////////////////////////////////////////////////
//
// In OnTimer we will use only the performance metrics macros.
//
//  For ease of use, a return macro will be defined. This is the 
//  only requirement for performance metrics to be displayed in the 
//  experts journal.
//
//  Performance macros are enabled only in release versions
//  of the code, they are disabled inside of debugging environments.
//
//  Debugging environment is defined by the flag LIB_DEBUG.
//  To enable performance macros, define LIB_PERF_PROFILING.
//
//  NOTE:
//      The macro "DBG_MSG_TRACE_RETURN" or "DBG_MSG_NOTRACE_RETURN"
//      must be used, so the performance control block will be properly closed.
//      Therefore we define relevant macros again, as shown in other examples.
//
//  Enable/disable tracing does not influence the perfomrance macros, as tracing 
//  is only enabled when "LIB_DEBUG" is defined and "PERF_*"-macros are disabled.
//  You will get performance measurements when the global macro "LIB_PERF_PROFILING"
//  is defined and the global macro "LIB_DEBUG" is not defined.
//
//
// We can now define additional, custom performance blocks on a global level.
//  These work accross functions and can be used to trace certain parts of the
//  code. - Each block has its own call counter, therefore it is not advised
//  to reuse the blocks for different code sections, except of course thats 
//  exactly what you want to measure.
//
// These blocks may overlap, if required. - It is necessary t ounderstand, 
//  when doing so, you will also measure the times each macro takes. The execution
//  of these macros would be part of your measurement.
//
//  See example below.
//

// Create additional performance counters
//  IDs can be specified freely
    PERF_COUNTER_DEFINE_ID(on_timer_for_loop);

    // Additional example (these are not used)
    PERF_COUNTER_DEFINE_ID(if_g_func);
    PERF_COUNTER_DEFINE_ID(call_f3_func);
    PERF_COUNTER_DEFINE_ID(printf_f1);
    PERF_COUNTER_DEFINE_ID(printf_f2);

// Performance block IDs can be created on a global scope as well 
//  as on function local scope and even in code blocks inside of 
//  functions scope.


//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
/////////////////////////////////////
// Function performance trace code
#define DBG_TRACE_EXPERT_MAIN_ONTIMER_RETURN        DBG_MSG_NOTRACE_RETURN

/////////////////////////////////////
void OnTimer()
{
PERF_COUNTER_BEGIN;

    // Measuring a loop
    double sum = NULL;

    PERF_COUNTER_BEGIN_ID(on_timer_for_loop);
    for(int cnt = NULL; (cnt < 1000000) && !_StopFlag; cnt++)
    {
        sum += 1.0;
    }
    PERF_COUNTER_END_ID(on_timer_for_loop);
        


    /////////////////////////////////////////////////////////////////////////////
    //
    // Here an example of possible nested performance blocks.
    // This example is very theoretical, but shows how blocks can be applied.
    //
    
    // Local scope
    {
        // These definitions can be on any scope in
        //  the hirarchy above this scope of the function all
        //  the way up to the global scope.
        PERF_COUNTER_DEFINE_ID(a_local_scope);
        PERF_COUNTER_DEFINE_ID(nested_level_1);
        PERF_COUNTER_DEFINE_ID(overlapping_block_a);
        PERF_COUNTER_DEFINE_ID(overlapping_block_b);

        
        // Begin measurement of most outer block
        PERF_COUNTER_BEGIN_ID(a_local_scope);
        
            // Have some code here (not mandatory)

            
            // Nested performance block 1
            PERF_COUNTER_BEGIN_ID(nested_level_1);
            
                // Have more code here
                

                // Overlapping execution block
                PERF_COUNTER_BEGIN_ID(overlapping_block_a);

                    // Maybe some code here for block A
                                

                // Open next block
                PERF_COUNTER_BEGIN_ID(overlapping_block_b);
                
                    // Have some code here for block A and B (shared code block)


                // Now closing block A
                PERF_COUNTER_END_ID(overlapping_block_a);

                    // Some code for block B


                // Now closing block B
                PERF_COUNTER_END_ID(overlapping_block_b);

                // Maybe more code here for nested level 1

            
            // Close nested block 1            
            PERF_COUNTER_END_ID(nested_level_1);

            // Optionally have code here as well....

            
        // Close most outer block
        PERF_COUNTER_END_ID(a_local_scope);    
    }


    // The above example will also measure the performance counters itself.
    //  Although they are held as short as possible, code-wise, it will 
    //  have some influence on the pure code thats being measured.
    //
    //  The impact can vary, depending on the memory-paging that happens 
    //  by accessing different areas of memory. - The variations will be 
    //  within the margins of error and therefore can (mostly) be ignored.
    //
    /////////////////////////////////////////////////////////////////////////////



    /////////////////////////////////////////////////////////////////////////////
    //
    // Standalone performance macros example
    //
    //  To measure certain parts of code in a standalone environment, 
    //  following macros are implemented to do this.
    //
    //  There are three variations of this macro available. Depending on 
    //  the return value of the function to be measured, either having
    //  a return value, a return object or not having a return value, you 
    //  need to use the appropiate macro.
    //
    //      PERF_COUNTER_TIMEIT_V   (V = void)
    //          This macro is used to measure functions or function sequences
    //          that do not have a return value.
    //      
    //      PERF_COUNTER_TIMEIT_R   (R = return)
    //          This macro will return the resulting value (return value)
    //          of the function being called, or their combination of such.
    //          For a better understanding, see the examples below.
    //
    //      PERF_COUNTER_TIMEIT_O   (O = Object)
    //          This macro will return the object (returned by the function)
    //          of the function being called.
    //
    //  NOTE:
    //
    //      PERF_COUNTER_TIMEIT_V may not be mentioned more than once per line of code.
    //      Internally the macro uses the __LINE__ macro for identifying the call, 
    //      therefore it is mandatory to have only one call per code line.
    //      Also be aware, if including this macro inside of another macro, all 
    //      statements will be on one line of code.
    //
    //      PERF_COUNTER_TIMEIT_R uses the complier macro __COUNTER__ to identify 
    //      unique its callings. Be aware, in case you rely on the sequence of
    //      counting provided by __COUNTER__ within your code. Each mentioning of
    //      the macro "PERF_COUNTER_TIMEIT_R" will increase the __COUNTER__ by one.
    //
    //      PERF_COUNTER_TIMEIT_O notes are same as for PERF_COUNTER_TIMEIT_R, with 
    //      one addition: The Copy-Constructor of the returned object will be called 
    //      twice. The first call contributes to the measurement results, the second 
    //      consequently does not. In order to encapusalte the function calls, an 
    //      additional copy of the object is required within the process of returning 
    //      values from the function.
    //
    //      For pointers as return value, either macro "PERF_COUNTER_TIMEIT_R" or
    //      "PERF_COUNTER_TIMEIT_O" can be used. - Both work the same way in this case.
    //

    // A single function call
    PERF_COUNTER_TIMEIT_V(
        printf("Some text A");
        );

    // A block sequence of function calls
    PERF_COUNTER_TIMEIT_V(
        printf("Some text B");
        printf("Some text C");
        );


    // Example of complex measurements
    b_start();


    // Return
    DBG_TRACE_EXPERT_MAIN_ONTIMER_RETURN;
}




int f1() { Sleep(100); return(1); }
int f2() { Sleep(200); return(2); }
int f3() { Sleep(300); return(3); }

int g()
{
  return(PERF_COUNTER_TIMEIT_R(f1() + f2() + f3() == 6) ?
                       PERF_COUNTER_TIMEIT_R(f1() * f2()) : 0);
}

//+------------------------------------------------------------------+
//| Service program start function                                   |
//+------------------------------------------------------------------+
void b_start()
{
    if (PERF_COUNTER_TIMEIT_R(g()) == 2)   
    { 
        PERF_COUNTER_TIMEIT_V(
            Print(f3() == 3); 
            );
    }

    PERF_COUNTER_TIMEIT_V(
        Print(f1());
        );

    PERF_COUNTER_TIMEIT_V(
        Print(f2());
        );

    return;
};







///////////////////////////////////////////////////////////////////////
//
// In call_sub_function we will use full tracing of the function call.
//
//  This example shows the difference for a function with 
//  a return value.
//
//  Take notice of the used macros. - It is required to use a 
//  different return macro for this type of function call.
//
//  Instead of using the macro "DBG_MSG_TRACE_RETURN" and "DBG_MSG_NOTRACE_RETURN"
//  here the macros "DBG_MSG_TRACE_RETURN_VAR(x)" and "DBG_MSG_NOTRACE_RETURN_VAR(x)" 
//  are used.
//
//  For use of objects of type "struct", "class" or "interface", a separate 
//  return value macro must be used. - (This implementation is new in MQL5)
//
//  Returning objects from functions, use following macros:
//      "DBG_MSG_TRACE_RETURN_OBJ(x)" and "DBG_MSG_NOTRACE_RETURN_OBJ(x)"
//
//  For use with pointers as return value, either macro can be used, preferably use
//      "DBG_MSG_TRACE_RETURN_VAR(x)".
//
//  IMPORTANT NOTICE:
//      Calling the macro with the parameter "NULL" requires!!! a cast operation.
//      (This functions return value is "int", "const int" or "int" doesnt make a difference)
//
//      Example:
//      DBG_MSG_TRACE_RETURN_VAR((int)NULL);
//
//      This cast operation can be included in the macro definition itself, like this:
//      #define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN(x) DBG_MSG_TRACE_RETURN_VAR((int)x)
//      #define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN(x) DBG_MSG_NOTRACE_RETURN_VAR(x)
//  
//      or inside the function using the custom defined macro:
//      DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN((int)NULL);
//
//      See second function example below.
//      Here the cast operation is inside the functions body.
//      NOTE: The runtime macro does not require the cast operation.
//
// Again the enbale trace macro needs to be defined, so tracing 
//  is enabled.

// Enable tracing of this function
#define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION 

//+------------------------------------------------------------------+
//| Example function with return value                               |
//+------------------------------------------------------------------+
/////////////////////////////////////
// Function debug trace code
#ifdef DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION
    #undef DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION(x) x
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN(x) DBG_MSG_TRACE_RETURN_VAR(x)

#else
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION(x)
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN(x) DBG_MSG_NOTRACE_RETURN_VAR(x)

#endif
/////////////////////////////////////
const int call_sub_function()
{
DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION(
    DBG_MSG_TRACE_BEGIN;    
    );
PERF_COUNTER_BEGIN;

    // Do some stuff
    if(MathRand() == 5)
    { DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN(5); }

    // Return
    DBG_TRACE_EXPERT_MAIN_CALL_SUB_FUNCTION_RETURN((int)NULL);
}







// Enable tracing of this function
#define DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION 

//+------------------------------------------------------------------+
//| Example function with return value                               |
//+------------------------------------------------------------------+
/////////////////////////////////////
// Function debug trace code
#ifdef DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION
    #undef DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION(x) x
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION_RETURN(x) DBG_MSG_TRACE_RETURN_VAR((int)x)

#else
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION(x)
    #define DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION_RETURN(x) DBG_MSG_NOTRACE_RETURN_VAR(x)

#endif
/////////////////////////////////////
const int call_sub2_function()
{
DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION(
    DBG_MSG_TRACE_BEGIN;    
    );
PERF_COUNTER_BEGIN;

    // Do some stuff
    if(MathRand() == 12)
    { DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION_RETURN(12); }

    // Return
    DBG_TRACE_EXPERT_MAIN_CALL_SUB2_FUNCTION_RETURN(NULL);
}






class CObj
{
    public:
    int test;
    
    CObj() :
        test(NULL)
    { };

    CObj(const CObj& p_in)
    { 
        test = p_in.test;
        printf("COPY");
    };

        
};


// Enable tracing of this function
#define DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION 

//+------------------------------------------------------------------+
//| Example function with return object                              |
//+------------------------------------------------------------------+
/////////////////////////////////////
// Function debug trace code
#ifdef DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION
    #undef DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION
    #define DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION(x) x
    #define DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION_RETURN(x) DBG_MSG_TRACE_RETURN_OBJ(x)

#else
    #define DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION(x)
    #define DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION_RETURN(x) DBG_MSG_NOTRACE_RETURN_OBJ(x)

#endif
/////////////////////////////////////
CObj call_obj_function()
{
DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION(
    DBG_MSG_TRACE_BEGIN;    
    );
PERF_COUNTER_BEGIN;

    // Local init
    CObj    test_obj;
    CObj*   test_ptr = new CObj();

    printf("1");
    CObj retval = PERF_COUNTER_TIMEIT_O(test_obj);

    printf("2");
    CObj* ret_ptr1 = PERF_COUNTER_TIMEIT_O(test_ptr);

    printf("3");
    CObj* ret_ptr2 = PERF_COUNTER_TIMEIT_R(test_ptr);

    printf("4");

    DBG_MSG_VAR(1);
    DBG_MSG_VAR(test_obj);
    DBG_MSG_VAR(test_ptr);

    // Do some stuff
    if(MathRand() == 12)
    { DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION_RETURN(test_obj); }

    // Return
    DBG_TRACE_EXPERT_MAIN_CALL_OBJ_FUNCTION_RETURN(test_obj);
}


*/